#include "db/NewDB.h"
#include "db/IndexDB.h"
#include "util/coding.h"
#include "db/write_batch_internal.h"
#include <string>
#include <sstream>
#include <unordered_map>
#include <iostream>

namespace leveldb{
NewDB::~NewDB() {
    // 必须显式实现析构函数
    // 如果有需要清理的资源，可以在这里做
}

Status NewDB::Open(const Options& options, const std::string& name, NewDB** dbptr) {
    // 打开底层的数据库
    DB* dataDB;
    Status s1 = DB::Open(options, name + "_data", &dataDB);
    if (!s1.ok()) {
        return s1;  // 如果打开失败，返回错误
    }

    DB* indexDB;
    Status s2 = DB::Open(options, name + "_index", &indexDB);
    if (!s2.ok()) {
        return s2;  // 如果打开失败，返回错误
    }
    // 创建一个 NewDB 实例
    *dbptr = new NewDB();

    // 初始化 data_db_ 和 index_db_
    (*dbptr)->data_db_ = std::unique_ptr<DB>(dataDB);  // 将打开的数据库指针传递给 data_db_
    (*dbptr)->index_db_ = std::unique_ptr<DB>(indexDB);  // 创建 IndexDB

    return Status::OK();  // 返回成功
}

FieldArray ParseFields(const std::string& value_str){
  FieldArray fields;
  const char* data = value_str.data();
  size_t length = value_str.size();

  while (length >= 4) {
    uint32_t field_size = DecodeFixed32(data);
    if (length < 4 + field_size) {
      break;
    }

    std::string field(data + 4, field_size);
    size_t colon_pos = field.find(':');

    std::string field_name = field.substr(0, colon_pos);
    std::string field_value = field.substr(colon_pos + 1);

    fields.push_back(std::make_pair(field_name,field_value));

    data += 4 + field_size;
    length -= 4 + field_size;
  }

  return fields;
}

bool NewDB::CreateIndexOnField(const std::string& field_name) {
    
    //测试用例现在还不全，以后还要写一个已经有多条数据记录的数据库，
    //然后用户创建索引的场景，所以创建索引的时候要对已有的数据遍历一遍。
    //这个和put字段的时候增加索引数据不矛盾的，都是需要的
    // 遍历 data_db_ 中的所有键值对
    leveldb::ReadOptions read_options;
    leveldb::WriteBatch batch;
    std::unique_ptr<leveldb::Iterator> it(data_db_->NewIterator(read_options));

    for (it->SeekToFirst(); it->Valid(); it->Next()) {
        // 获取主数据库的键值对
        std::string key = it->key().ToString();
        std::string value = it->value().ToString();

        // 解析值，提取字段数组
        FieldArray fields = ParseFields(value);

        // 查找指定字段
        for (const auto& field : fields) {
            if (field.first == field_name) {
                // 构造索引键
                std::string index_key = ConstructKey(key, field);

                // 插入到索引数据库
                
                batch.Put(index_key, Slice());
            }
        }
    }

    // 如果迭代器出错
    if (!it->status().ok()) {
        // 从头开始创建，覆盖掉原本失效的东西
        return false;
    }
    // 批量写入
    WriteOptions write_options;
    leveldb::Status status = index_db_->Write(write_options, &batch);
    if (!status.ok()) {
        return false;
    }
    indexed_fields_.insert(field_name);
    return true;
}

// 这两个函数如果并行的话一致性怎么保证
Status NewDB::Put_fields(const WriteOptions& options, const Slice& key, const FieldArray& fields){
    // std::lock_guard<std::mutex> lock(db_mutex_);  // 确保线程安全
    FieldArray current_fields;
    Status s = Get_fields(leveldb::ReadOptions(), key, &current_fields);
    // 在应用层面模拟事务处理    
    if(!s.ok()){
        // 1. 插入主数据到 DataDB
        Status data_status = data_db_->Put_fields(options, key, fields);
        if (!data_status.ok()) {
            return data_status; // 主数据写入失败，直接返回
        }        
        // 2. 遍历字段，检查是否需要建立索引
        // 要改成writebatch？
        // 然后加锁，插入datadb和插入indexdb的顺序应该不影响（如果操作的是同一条数据）
        for (const auto& field : fields) {
            // 如果字段名在 indexed_fields_ 中，才插入二级索引
            if (indexed_fields_.find(field.first) != indexed_fields_.end()) {
                // 构造索引的key，结构：FieldName_FieldValue_Key
                std::string index_key = ConstructKey(key, field);

                // 插入到 IndexDB
                Status index_status = index_db_->Put(options, index_key, Slice());  // value 是空的
                if (!index_status.ok()) {
                    // 回滚已写入的主数据
                    data_db_->Delete(options, key);
                    return index_status; // 返回索引写入错误
                }
            }
        }

        return Status::OK();
    }
    else{
        // std::cout << "start update...";

        data_db_->Put_fields(options, key, fields);

        return UpdateIndex(options, key, fields, current_fields);
    }

    return s;
}

Status NewDB::UpdateIndex(const WriteOptions& options, const Slice& key, const FieldArray& fields, const FieldArray& current_fields){
    // 构建current_fields的映射
    std::unordered_map<std::string, std::string> current_map;
    for (const auto& field : current_fields) {
        current_map[field.first] = field.second;
    }

    // 构建fields的映射
    std::unordered_map<std::string, std::string> fields_map;
    for (const auto& field : fields) {
        fields_map[field.first] = field.second;
    }

    // 删除的字段
    FieldArray deleted_fields;
    // 新增的字段
    FieldArray insert_fields;

    // fields中的字段
    for (const auto& field : fields) {
        // fields中存在但current_fields中不存在
        if (current_map.find(field.first) == current_map.end()) {
            insert_fields.push_back(field);
        } else if (current_map[field.first] != field.second) {
            // current_fields中存在但是值不同
            insert_fields.push_back(field);
            deleted_fields.push_back(std::make_pair(field.first, current_map[field.first]));
        }
    }

    // current_fields中的字段
    for (const auto& field : current_fields) {
        if (fields_map.find(field.first) == fields_map.end()) {
            deleted_fields.push_back(field);
        }
    }    

    for (const auto& field : insert_fields) {
        if (indexed_fields_.find(field.first) != indexed_fields_.end()) {
            // std::cout << field.first << field.second << std::endl;

            std::string index_key = ConstructKey(key, field);
            index_db_->Put(options, index_key, Slice());
        }
    }

    for (const auto& field : deleted_fields) {
        if (indexed_fields_.find(field.first) != indexed_fields_.end()) {
            // std::cout << field.first << field.second << std::endl;
            std::string index_key = ConstructKey(key, field);
            // std::cout << "delete " << index_key << std::endl;
            Status s = index_db_->Delete(leveldb::WriteOptions(), Slice(index_key));
            if(!s.ok()){
                return s;
            }
        }
    }
    return Status::OK();
}

Status NewDB::Get_fields(const ReadOptions& options, const Slice& key, FieldArray* fields){
    //get value(fields) with key
    // 从 DataDB 获取数据
    // std::string value_str;
    // Status s = data_db_->Get(options, key, &value_str);
    Status s = data_db_->Get_fields(options, key, fields);   
    if (!s.ok()) {
        return s;  // 如果获取失败，返回状态
    }   
    return Status::OK();
}

bool NewDB::Delete(const WriteOptions& options, const Slice& key){  
    FieldArray fields;
    data_db_->Get_fields(leveldb::ReadOptions(), key, &fields);

    Status s = data_db_->Delete(options, key);
    if(!s.ok()){
        return false;
    }  

    for(auto& field:fields){
        if(indexed_fields_.find(field.first) != indexed_fields_.end()){   //该字段在已有索引的集合中
            // bool deleted = DeleteIndex(field.first);
            std::string index_key = ConstructKey(key, field);
            Status s = index_db_->Delete(options, Slice(index_key));
            if(!s.ok()){
                return false;
            }
        }
    }
    return true;
}

std::vector<std::string> NewDB::FindKeysByField(leveldb::DB* db, Field &field){  
    //get keys with field
    std::vector<std::string> matching_keys;
    matching_keys = data_db_->FindKeysByField(db, field);
    return matching_keys;
}

// 构造索引的key，结构：FieldName_FieldValue_Key-橙
std::string NewDB::ConstructKey(const Slice& key, const Field& field){
    std::ostringstream oss;
    oss << field.first << ":" << field.second << "_" << key.ToString();  
    return oss.str(); 
}

// 从索引键中提取原数据的键
std::string NewDB::ExtractKey(const Slice& key){
    //extract key of dataDB from key of indexDB
    std::string fullKey(key.data(), key.size());
    // size_t pos1 = fullKey.find('_');
    size_t pos1 = fullKey.find('_');   
    return fullKey.substr(pos1 + 1);  // 提取 'Key' 部分
}

std::vector<std::string> NewDB::QueryByIndex(Field &field){   
    std::vector<std::string> matching_keys;
    std::string prefix = ConstructKey(Slice(), field);   //前缀-字段名+字段值
    leveldb::Iterator* iter = index_db_->NewIterator(leveldb::ReadOptions());
    for(iter->Seek(prefix); iter->Valid() && iter->key().starts_with(prefix); iter->Next()){
        Slice index_key = iter->key();
        std::string data_key = ExtractKey(index_key);  //解析索引键中的key
        matching_keys.push_back(data_key);
    }
    delete iter;
    return matching_keys;
}

bool NewDB::DeleteIndex(const std::string& field_name){    
    indexed_fields_.erase(field_name);
    std::string prefix = field_name;
    leveldb::Iterator* iter = index_db_->NewIterator(leveldb::ReadOptions());
    for(iter->Seek(prefix); iter->Valid() && iter->key().starts_with(prefix); iter->Next()){
        Slice index_key = iter->key();
        Status s = index_db_->Delete(leveldb::WriteOptions(), index_key);  //删除每个符合字段名的key
        if(!s.ok()){
            return false;
        }
    }
    delete iter;
    return true;
}

}





